@@ -18,13 +18,13 @@
 ##
 
 import os
-import time
 import base64
 import sha
 import socket
 import sys
 
-from time import localtime, strftime, gmtime
+from time import (altzone, daylight, gmtime, localtime, mktime, strftime,
+	time as time_time, timezone, tzname)
 from calendar import timegm
 
 import socks5
@@ -34,6 +34,7 @@ from common import GnuPG
 from common import helpers
 from common import gajim
 from common import atom
+from common import exceptions
 from common.commands import ConnectionCommands
 from common.pubsub import ConnectionPubSub
 
@@ -327,7 +328,7 @@ class ConnectionBytestream:
 		iq.setID(auth_id)
 		query = iq.setTag('query')
 		query.setNamespace(common.xmpp.NS_BYTESTREAM)
-		query.setAttr('sid',  proxy['sid'])
+		query.setAttr('sid', proxy['sid'])
 		activate = query.setTag('activate')
 		activate.setData(file_props['proxy_receiver'])
 		iq.setID(auth_id)
@@ -395,7 +396,7 @@ class ConnectionBytestream:
 
 	def _ResultCB(self, con, iq_obj):
 		gajim.log.debug('_ResultCB')
-		# if we want to respect jep-0065 we have to check for proxy
+		# if we want to respect xep-0065 we have to check for proxy
 		# activation result in any result iq
 		real_id = unicode(iq_obj.getAttr('id'))
 		if real_id[:3] != 'au_':
@@ -418,7 +419,7 @@ class ConnectionBytestream:
 		gajim.proxy65_manager.resolve_result(frm, query)
 		
 		try:
-			streamhost =  query.getTag('streamhost-used')
+			streamhost = query.getTag('streamhost-used')
 		except: # this bytestream result is not what we need
 			pass
 		id = real_id[3:]
@@ -569,12 +570,12 @@ class ConnectionBytestream:
 class ConnectionDisco:
 	''' hold xmpppy handlers and public methods for discover services'''
 	def discoverItems(self, jid, node = None, id_prefix = None):
-		'''According to JEP-0030: jid is mandatory,
+		'''According to XEP-0030: jid is mandatory,
 		name, node, action is optional.'''
 		self._discover(common.xmpp.NS_DISCO_ITEMS, jid, node, id_prefix)
 
 	def discoverInfo(self, jid, node = None, id_prefix = None):
-		'''According to JEP-0030:
+		'''According to XEP-0030:
 			For identity: category, type is mandatory, name is optional.
 			For feature: var is mandatory'''
 		self._discover(common.xmpp.NS_DISCO_INFO, jid, node, id_prefix)
@@ -591,37 +592,15 @@ class ConnectionDisco:
 			agent)
 		self.connection.SendAndCallForResponse(iq, self._ReceivedRegInfo,
 			{'agent': agent})
-	
-	def build_data_from_dict(self, query, config):
-		x = query.setTag(common.xmpp.NS_DATA + ' x', attrs = {'type': 'submit'})
-		i = 0
-		while config.has_key(i):
-			if not config[i].has_key('type'):
-				i += 1
-				continue
-			if config[i]['type'] == 'fixed':
-				i += 1
-				continue
-			tag = x.addChild('field')
-			if config[i].has_key('var'):
-				tag.setAttr('var', config[i]['var'])
-			if config[i].has_key('values'):
-				for val in config[i]['values']:
-					if val == False:
-						val = '0'
-					elif val == True:
-						val = '1'
-					# Force to create a new child
-					tag.addChild('value').addData(val)
-			i += 1
-	
+
 	def register_agent(self, agent, info, is_form = False):
 		if not self.connection:
 			return
 		if is_form:
 			iq = common.xmpp.Iq('set', common.xmpp.NS_REGISTER, to = agent)
 			query = iq.getTag('query')
-			self.build_data_from_dict(query, info)
+			info.setAttr('type', 'submit')
+			query.addChild(node = info)
 			self.connection.send(iq)
 		else:
 			# fixed: blocking
@@ -652,22 +631,12 @@ class ConnectionDisco:
 			common.xmpp.NS_DISCO, frm = to)
 		iq.setAttr('id', id)
 		query = iq.setTag('query')
-		# bytestream transfers
-		feature = common.xmpp.Node('feature')
-		feature.setAttr('var', common.xmpp.NS_BYTESTREAM)
-		query.addChild(node=feature)
-		# si methods
-		feature = common.xmpp.Node('feature')
-		feature.setAttr('var', common.xmpp.NS_SI)
-		query.addChild(node=feature)
-		# filetransfers transfers
-		feature = common.xmpp.Node('feature')
-		feature.setAttr('var', common.xmpp.NS_FILE)
-		query.addChild(node=feature)
-		# exposing adhoc commands
-		feature = common.xmpp.Node('feature')
-		feature.setAttr('var', common.xmpp.NS_COMMANDS)
-		query.addChild(node=feature)
+		query.setAttr('node','http://gajim.org/caps#' + gajim.version)
+		for f in (common.xmpp.NS_BYTESTREAM, common.xmpp.NS_SI, \
+						common.xmpp.NS_FILE, common.xmpp.NS_COMMANDS):
+			feature = common.xmpp.Node('feature')
+			feature.setAttr('var', f)
+			query.addChild(node=feature)
 		
 		self.connection.send(iq)
 		raise common.xmpp.NodeProcessed
@@ -715,6 +684,10 @@ class ConnectionDisco:
 	def _DiscoverItemsGetCB(self, con, iq_obj):
 		gajim.log.debug('DiscoverItemsGetCB')
 		node = iq_obj.getTagAttr('query', 'node')
+		if node is None:
+			result = iq_obj.buildReply('result')
+			self.connection.send(result)
+			raise common.xmpp.NodeProcessed
 		if node==common.xmpp.NS_COMMANDS:
 			self.commandListQuery(con, iq_obj)
 			raise common.xmpp.NodeProcessed
@@ -730,6 +703,8 @@ class ConnectionDisco:
 		else:
 			iq = iq_obj.buildReply('result')
 			q = iq.getTag('query')
+			if node:
+				q.setAttr('node', node)
 			q.addChild('identity', attrs = {'type': 'pc', 'category': 'client',
 				'name': 'Gajim'})
 			extension = None
@@ -737,18 +712,23 @@ class ConnectionDisco:
 				extension = node[node.index('#') + 1:]
 			client_version = 'http://gajim.org/caps#' + gajim.version
 
-			if node in (None, client_version) or extension == 'ftrans':
+			if node in (None, client_version):
 				q.addChild('feature', attrs = {'var': common.xmpp.NS_BYTESTREAM})
 				q.addChild('feature', attrs = {'var': common.xmpp.NS_SI})
 				q.addChild('feature', attrs = {'var': common.xmpp.NS_FILE})
-			if node in (None, client_version) or extension == 'xhtml':
-				q.addChild('feature', attrs = {'var': common.xmpp.NS_XHTML_IM})
-			if node in (None, client_version):
 				q.addChild('feature', attrs = {'var': common.xmpp.NS_MUC})
-				q.addChild('feature', attrs = {'var': common.xmpp.NS_CHATSTATES})
 				q.addChild('feature', attrs = {'var': common.xmpp.NS_COMMANDS})
 				q.addChild('feature', attrs = {'var': common.xmpp.NS_DISCO_INFO})
 
+			if (node is None or extension == 'cstates') and gajim.config.get('outgoing_chat_state_notifactions') != 'disabled':
+				q.addChild('feature', attrs = {'var': common.xmpp.NS_CHATSTATES})
+
+			if (node is None or extension == 'xhtml') and not gajim.config.get('ignore_incoming_xhtml'):
+				q.addChild('feature', attrs = {'var': common.xmpp.NS_XHTML_IM})
+
+			if node is None:
+				q.addChild('feature', attrs = {'var': common.xmpp.NS_TIME_REVISED})
+
 			if q.getChildren():
 				self.connection.send(iq)
 				raise common.xmpp.NodeProcessed
@@ -760,7 +740,7 @@ class ConnectionDisco:
 
 	def _DiscoverInfoCB(self, con, iq_obj):
 		gajim.log.debug('DiscoverInfoCB')
-		# According to JEP-0030:
+		# According to XEP-0030:
 		# For identity: category, type is mandatory, name is optional.
 		# For feature: var is mandatory
 		identities, features, data = [], [], []
@@ -816,7 +796,7 @@ class ConnectionVcard:
 		self.vcard_sha = None
 		self.vcard_shas = {} # sha of contacts
 		self.room_jids = [] # list of gc jids so that vcard are saved in a folder
-		
+
 	def add_sha(self, p, send_caps = True):
 		c = p.setTag('x', namespace = common.xmpp.NS_VCARD_UPDATE)
 		if self.vcard_sha is not None:
@@ -826,10 +806,17 @@ class ConnectionVcard:
 		return p
 	
 	def add_caps(self, p):
-		''' advertise our capabilities in presence stanza (jep-0115)'''
+		''' advertise our capabilities in presence stanza (xep-0115)'''
 		c = p.setTag('c', namespace = common.xmpp.NS_CAPS)
 		c.setAttr('node', 'http://gajim.org/caps')
-		c.setAttr('ext', 'ftrans xhtml')
+		ext = []
+		if not gajim.config.get('ignore_incoming_xhtml'):
+			ext.append('xhtml')
+		if gajim.config.get('outgoing_chat_state_notifactions') != 'disabled':
+			ext.append('cstates')
+ 
+		if len(ext):
+			c.setAttr('ext', ' '.join(ext))
 		c.setAttr('ver', gajim.version)
 		return p
 	
@@ -869,9 +856,12 @@ class ConnectionVcard:
 			path_to_file = os.path.join(gajim.VCARD_PATH, puny_jid, puny_nick)
 		else:
 			path_to_file = path
-		fil = open(path_to_file, 'w')
-		fil.write(str(card))
-		fil.close()
+		try:
+			fil = open(path_to_file, 'w')
+			fil.write(str(card))
+			fil.close()
+		except IOError, e:
+			self.dispatch('ERROR', (_('Disk Write Error'), str(e)))
 	
 	def get_cached_vcard(self, fjid, is_fake_jid = False):
 		'''return the vcard as a dict
@@ -904,9 +894,10 @@ class ConnectionVcard:
 		vcard['resource'] = gajim.get_resource_from_jid(fjid)
 		return vcard
 
-	def request_vcard(self, jid = None, is_fake_jid = False):
-		'''request the VCARD. If is_fake_jid is True, it means we request a vcard
-		to a fake jid, like in private messages in groupchat'''
+	def request_vcard(self, jid = None, groupchat_jid = None):
+		'''request the VCARD. If groupchat_jid is not nul, it means we request a vcard
+		to a fake jid, like in private messages in groupchat. jid can be the
+		real jid of the contact, but we want to consider it comes from a fake jid'''
 		if not self.connection:
 			return
 		iq = common.xmpp.Iq(typ = 'get')
@@ -919,13 +910,13 @@ class ConnectionVcard:
 		j = jid
 		if not j:
 			j = gajim.get_jid_from_account(self.name)
-		self.awaiting_answers[id] = (VCARD_ARRIVED, j)
-		if is_fake_jid:
-			room_jid, nick = gajim.get_room_and_nick_from_fjid(jid)
+		self.awaiting_answers[id] = (VCARD_ARRIVED, j, groupchat_jid)
+		if groupchat_jid:
+			room_jid, nick = gajim.get_room_and_nick_from_fjid(groupchat_jid)
 			if not room_jid in self.room_jids:
 				self.room_jids.append(room_jid)
+			self.groupchat_jids[id] = groupchat_jid
 		self.connection.send(iq)
-			#('VCARD', {entry1: data, entry2: {entry21: data, ...}, ...})
 
 	def send_vcard(self, vcard):
 		if not self.connection:
@@ -1008,24 +999,29 @@ class ConnectionVcard:
 			# If vcard is empty, we send to the interface an empty vcard so that
 			# it knows it arrived
 			jid = self.awaiting_answers[id][1]
+			groupchat_jid = self.awaiting_answers[id][2]
+			frm = jid
+			if groupchat_jid:
+				# We do as if it comes from the fake_jid
+				frm = groupchat_jid
 			our_jid = gajim.get_jid_from_account(self.name)
 			if iq_obj.getType() == 'error' and jid == our_jid:
 				# our server doesn't support vcard
 				self.vcard_supported = False
 			if not iq_obj.getTag('vCard') or iq_obj.getType() == 'error':
-				if jid and jid != our_jid:
+				if frm and frm != our_jid:
 					# Write an empty file
-					self.save_vcard_to_hd(jid, '')
-					self.dispatch('VCARD', {'jid': jid})
-				elif jid == our_jid:
-					self.dispatch('MYVCARD', {'jid': jid})
+					self.save_vcard_to_hd(frm, '')
+					self.dispatch('VCARD', {'jid': frm})
+				elif frm == our_jid:
+					self.dispatch('MYVCARD', {'jid': frm})
 		elif self.awaiting_answers[id][0] == AGENT_REMOVED:
 			jid = self.awaiting_answers[id][1]
 			self.dispatch('AGENT_REMOVED', jid)
 		elif self.awaiting_answers[id][0] == METACONTACTS_ARRIVED:
 			if iq_obj.getType() == 'result':
 				# Metacontact tags
-				# http://www.jabber.org/jeps/jep-XXXX.html
+				# http://www.xmpp.org/extensions/xep-0209.html
 				meta_list = {}
 				query = iq_obj.getTag('query')
 				storage = query.getTag('storage')
@@ -1035,6 +1031,10 @@ class ConnectionVcard:
 					tag = meta.getAttr('tag')
 					data = {'jid': jid}
 					order = meta.getAttr('order')
+					try:
+						order = int(order)
+					except:
+						order = 0
 					if order != None:
 						data['order'] = order
 					if meta_list.has_key(tag):
@@ -1043,7 +1043,7 @@ class ConnectionVcard:
 						meta_list[tag] = [data]
 				self.dispatch('METACONTACTS', meta_list)
 			else:
-				self.metacontacts_supported = False
+				self.private_storage_supported = False
 			# We can now continue connection by requesting the roster
 			self.connection.initRoster()
 		elif self.awaiting_answers[id][0] == PRIVACY_ARRIVED:
@@ -1061,10 +1061,15 @@ class ConnectionVcard:
 			return
 		if not vc.getTag('vCard').getNamespace() == common.xmpp.NS_VCARD:
 			return
+		id = vc.getID()
 		frm_iq = vc.getFrom()
 		our_jid = gajim.get_jid_from_account(self.name)
 		resource = ''
-		if frm_iq:
+		if id in self.groupchat_jids:
+			who = self.groupchat_jids[id]
+			frm, resource = gajim.get_room_and_nick_from_fjid(who)
+			del self.groupchat_jids[id]
+		elif frm_iq:
 			who = helpers.get_full_jid_from_iq(vc)
 			frm, resource = gajim.get_room_and_nick_from_fjid(who)
 		else:
@@ -1136,6 +1141,7 @@ class ConnectionVcard:
 			p = self.add_sha(p)
 			self.connection.send(p)
 		else:
+			#('VCARD', {entry1: data, entry2: {entry21: data, ...}, ...})
 			self.dispatch('VCARD', vcard)
 
 class ConnectionHandlers(ConnectionVcard, ConnectionBytestream, ConnectionDisco, ConnectionCommands, ConnectionPubSub):
@@ -1177,7 +1183,8 @@ class ConnectionHandlers(ConnectionVcard, ConnectionBytestream, ConnectionDisco,
 			id = iq_obj.getTagAttr('confirm', 'id')
 			method = iq_obj.getTagAttr('confirm', 'method')
 			url = iq_obj.getTagAttr('confirm', 'url')
-			self.dispatch('HTTP_AUTH', (method, url, id, iq_obj));
+			msg = iq_obj.getTagData('body') # In case it's a message with a body
+			self.dispatch('HTTP_AUTH', (method, url, id, iq_obj, msg));
 		raise common.xmpp.NodeProcessed
 
 	def _ErrorCB(self, con, iq_obj):
@@ -1195,7 +1202,7 @@ class ConnectionHandlers(ConnectionVcard, ConnectionBytestream, ConnectionDisco,
 	
 	def _PrivateCB(self, con, iq_obj):
 		'''
-		Private Data (JEP 048 and 049)
+		Private Data (XEP 048 and 049)
 		'''
 		gajim.log.debug('PrivateCB')
 		query = iq_obj.getTag('query')
@@ -1204,7 +1211,7 @@ class ConnectionHandlers(ConnectionVcard, ConnectionBytestream, ConnectionDisco,
 			ns = storage.getNamespace()
 			if ns == 'storage:bookmarks':
 				# Bookmarked URLs and Conferences
-				# http://www.jabber.org/jeps/jep-0048.html
+				# http://www.xmpp.org/extensions/xep-0048.html
 				confs = storage.getTags('conference')
 				for conf in confs:
 					autojoin_val = conf.getAttr('autojoin')
@@ -1225,7 +1232,7 @@ class ConnectionHandlers(ConnectionVcard, ConnectionBytestream, ConnectionDisco,
 
 			elif ns == 'gajim:prefs':
 				# Preferences data
-				# http://www.jabber.org/jeps/jep-0049.html
+				# http://www.xmpp.org/extensions/xep-0049.html
 				#TODO: implement this
 				pass
 			elif ns == 'storage:rosternotes':
@@ -1244,18 +1251,18 @@ class ConnectionHandlers(ConnectionVcard, ConnectionBytestream, ConnectionDisco,
 		if storage_tag:
 			ns = storage_tag.getNamespace()
 			if ns == 'storage:metacontacts':
-				self.metacontacts_supported = False
-				# Private XML Storage (JEP49) is not supported by server
+				self.private_storage_supported = False
+				# Private XML Storage (XEP49) is not supported by server
 				# Continue connecting
 				self.connection.initRoster()
 
 	def _rosterSetCB(self, con, iq_obj):
 		gajim.log.debug('rosterSetCB')
 		for item in iq_obj.getTag('query').getChildren():
-			jid  = helpers.parse_jid(item.getAttr('jid'))
+			jid = helpers.parse_jid(item.getAttr('jid'))
 			name = item.getAttr('name')
-			sub  = item.getAttr('subscription')
-			ask  = item.getAttr('ask')
+			sub = item.getAttr('subscription')
+			ask = item.getAttr('ask')
 			groups = []
 			for group in item.getTags('group'):
 				groups.append(group.getData())
@@ -1295,7 +1302,12 @@ class ConnectionHandlers(ConnectionVcard, ConnectionBytestream, ConnectionDisco,
 			seconds = int(seconds)
 		except:
 			return
-		who = helpers.get_full_jid_from_iq(iq_obj)
+		id = iq_obj.getID()
+		if id in self.groupchat_jids:
+			who = self.groupchat_jids[id]
+			del self.groupchat_jids[id]
+		else:
+			who = helpers.get_full_jid_from_iq(iq_obj)
 		jid_stripped, resource = gajim.get_room_and_nick_from_fjid(who)
 		self.dispatch('LAST_STATUS_TIME', (jid_stripped, resource, seconds, status))
 	
@@ -1310,7 +1322,12 @@ class ConnectionHandlers(ConnectionVcard, ConnectionBytestream, ConnectionDisco,
 			client_info += ' ' + qp.getTag('version').getData()
 		if qp.getTag('os'):
 			os_info += qp.getTag('os').getData()
-		who = helpers.get_full_jid_from_iq(iq_obj)
+		id = iq_obj.getID()
+		if id in self.groupchat_jids:
+			who = self.groupchat_jids[id]
+			del self.groupchat_jids[id]
+		else:
+			who = helpers.get_full_jid_from_iq(iq_obj)
 		jid_stripped, resource = gajim.get_room_and_nick_from_fjid(who)
 		self.dispatch('OS_INFO', (jid_stripped, resource, client_info, os_info))
 
@@ -1318,18 +1335,21 @@ class ConnectionHandlers(ConnectionVcard, ConnectionBytestream, ConnectionDisco,
 		gajim.log.debug('TimeCB')
 		iq_obj = iq_obj.buildReply('result')
 		qp = iq_obj.getTag('query')
-		qp.setTagData('utc', strftime("%Y%m%dT%T", gmtime()))
-		qp.setTagData('tz', strftime("%Z", gmtime()))
-		qp.setTagData('display', strftime("%c", localtime()))
+		qp.setTagData('utc', strftime('%Y%m%dT%T', gmtime()))
+		qp.setTagData('tz', tzname[daylight])
+		qp.setTagData('display', strftime('%c', localtime()))
 		self.connection.send(iq_obj)
 		raise common.xmpp.NodeProcessed
 
 	def _TimeRevisedCB(self, con, iq_obj):
 		gajim.log.debug('TimeRevisedCB')
 		iq_obj = iq_obj.buildReply('result')
-		qp = iq_obj.setTag('time')
-		qp.setTagData('utc', strftime("%Y-%m-%dT%TZ", gmtime()))
-		qp.setTagData('tzo', "%+03d:00"% (-time.timezone/(60*60)))
+		qp = iq_obj.setTag('time',
+			namespace=common.xmpp.NS_TIME_REVISED)
+		qp.setTagData('utc', strftime('%Y-%m-%dT%TZ', gmtime()))
+		zone = -(timezone, altzone)[daylight] / 60
+		tzo = (zone / 60, abs(zone % 60))
+		qp.setTagData('tzo', '%+03d:%02d' % (tzo))
 		self.connection.send(iq_obj)
 		raise common.xmpp.NodeProcessed
 
@@ -1381,13 +1401,18 @@ class ConnectionHandlers(ConnectionVcard, ConnectionBytestream, ConnectionDisco,
 		if msg.getTag('event') is not None:
 			self._pubsubEventCB(con, msg)
 			return
+		# check if the message is a xep70-confirmation-request
+		if msg.getTag('confirm') and msg.getTag('confirm').namespace == \
+		common.xmpp.NS_HTTP_AUTH:
+			self._HttpAuthCB(con, msg)
+			return
 		msgtxt = msg.getBody()
 		msghtml = msg.getXHTML()
 		mtype = msg.getType()
 		subject = msg.getSubject() # if not there, it's None
 		tim = msg.getTimestamp()
-		tim = time.strptime(tim, '%Y%m%dT%H:%M:%S')
-		tim = time.localtime(timegm(tim))
+		tim = helpers.datetime_tuple(tim)
+		tim = localtime(timegm(tim))
 		frm = helpers.get_full_jid_from_iq(msg)
 		jid = helpers.get_jid_from_iq(msg)
 		no_log_for = gajim.config.get_per('accounts', self.name,
@@ -1407,10 +1432,10 @@ class ConnectionHandlers(ConnectionVcard, ConnectionBytestream, ConnectionDisco,
 				invite = None
 		delayed = msg.getTag('x', namespace = common.xmpp.NS_DELAY) != None
 		msg_id = None
-		composing_jep = None
+		composing_xep = None
 		# FIXME: Msn transport (CMSN1.2.1 and PyMSN0.10) do NOT RECOMMENDED
 		# invitation
-		# stanza (MUC JEP) remove in 2007, as we do not do NOT RECOMMENDED
+		# stanza (MUC XEP) remove in 2007, as we do not do NOT RECOMMENDED
 		xtags = msg.getTags('x')
 		for xtag in xtags:
 			if xtag.getNamespace() == common.xmpp.NS_CONFERENCE and not invite:
@@ -1419,22 +1444,22 @@ class ConnectionHandlers(ConnectionVcard, ConnectionBytestream, ConnectionDisco,
 				return
 		# chatstates - look for chatstate tags in a message if not delayed
 		if not delayed:
-			composing_jep = False
+			composing_xep = False
 			children = msg.getChildren()
 			for child in children:
 				if child.getNamespace() == 'http://jabber.org/protocol/chatstates':
 					chatstate = child.getName()
-					composing_jep = 'JEP-0085'
+					composing_xep = 'XEP-0085'
 					break
-			# No JEP-0085 support, fallback to JEP-0022
+			# No XEP-0085 support, fallback to XEP-0022
 			if not chatstate:
 				chatstate_child = msg.getTag('x', namespace = common.xmpp.NS_EVENT)
 				if chatstate_child:
 					chatstate = 'active'
-					composing_jep = 'JEP-0022'
+					composing_xep = 'XEP-0022'
 					if not msgtxt and chatstate_child.getTag('composing'):
 						chatstate = 'composing'
-		# JEP-0172 User Nickname
+		# XEP-0172 User Nickname
 		user_nick = msg.getTagData('nick')
 		if not user_nick:
 			user_nick = ''
@@ -1455,8 +1480,11 @@ class ConnectionHandlers(ConnectionVcard, ConnectionBytestream, ConnectionDisco,
 				error_msg = msgtxt
 				msgtxt = None
 			if self.name not in no_log_for:
-				gajim.logger.write('error', frm, error_msg, tim = tim,
-					subject = subject)
+				try:
+					gajim.logger.write('error', frm, error_msg, tim = tim,
+						subject = subject)
+				except exceptions.PysqliteOperationalError, e:
+					self.dispatch('ERROR', (_('Disk Write Error'), str(e)))
 			self.dispatch('MSGERROR', (frm, msg.getErrorCode(), error_msg, msgtxt,
 				tim))
 			return
@@ -1473,17 +1501,23 @@ class ConnectionHandlers(ConnectionVcard, ConnectionBytestream, ConnectionDisco,
 				if not self.last_history_line.has_key(jid):
 					return
 				self.dispatch('GC_MSG', (frm, msgtxt, tim, has_timestamp, msghtml))
-				if self.name not in no_log_for and not int(float(time.mktime(tim)))\
+				if self.name not in no_log_for and not int(float(mktime(tim)))\
 				<= self.last_history_line[jid] and msgtxt:
-					gajim.logger.write('gc_msg', frm, msgtxt, tim = tim)
+					try:
+						gajim.logger.write('gc_msg', frm, msgtxt, tim = tim)
+					except exceptions.PysqliteOperationalError, e:
+						self.dispatch('ERROR', (_('Disk Write Error'), str(e)))
 			return
 		elif mtype == 'chat': # it's type 'chat'
 			if not msg.getTag('body') and chatstate is None: #no <body>
 				return
 			if msg.getTag('body') and self.name not in no_log_for and jid not in\
 				no_log_for and msgtxt:
-				msg_id = gajim.logger.write('chat_msg_recv', frm, msgtxt, tim = tim,
-					subject = subject)
+				try:
+					msg_id = gajim.logger.write('chat_msg_recv', frm, msgtxt,
+						tim = tim, subject = subject)
+				except exceptions.PysqliteOperationalError, e:
+					self.dispatch('ERROR', (_('Disk Write Error'), str(e)))
 		else: # it's single message
 			if invite is not None:
 				item = invite.getTag('invite')
@@ -1494,14 +1528,17 @@ class ConnectionHandlers(ConnectionVcard, ConnectionBytestream, ConnectionDisco,
 				self.dispatch('GC_INVITATION',(frm, jid_from, reason, password))
 				return
 			if self.name not in no_log_for and jid not in no_log_for and msgtxt:
-				gajim.logger.write('single_msg_recv', frm, msgtxt, tim = tim,
-					subject = subject)
+				try:
+					gajim.logger.write('single_msg_recv', frm, msgtxt, tim = tim,
+						subject = subject)
+				except exceptions.PysqliteOperationalError, e:
+					self.dispatch('ERROR', (_('Disk Write Error'), str(e)))
 			mtype = 'normal'
 		treat_as = gajim.config.get('treat_incoming_messages')
 		if treat_as:
 			mtype = treat_as
 		self.dispatch('MSG', (frm, msgtxt, tim, encrypted, mtype,
-			subject, chatstate, msg_id, composing_jep, user_nick, msghtml))
+			subject, chatstate, msg_id, composing_xep, user_nick, msghtml))
 	# END messageCB
 
 	def _pubsubEventCB(self, con, msg):
@@ -1535,6 +1572,9 @@ class ConnectionHandlers(ConnectionVcard, ConnectionBytestream, ConnectionDisco,
 		ptype = prs.getType()
 		if ptype == 'available':
 			ptype = None
+		rfc_types = ('unavailable', 'error', 'subscribe', 'subscribed', 'unsubscribe', 'unsubscribed')
+		if ptype and not ptype in rfc_types:
+			ptype = None
 		gajim.log.debug('PresenceCB: %s' % ptype)
 		try:
 			who = helpers.get_full_jid_from_iq(prs)
@@ -1552,10 +1592,11 @@ class ConnectionHandlers(ConnectionVcard, ConnectionBytestream, ConnectionDisco,
 		is_gc = False # is it a GC presence ?
 		sigTag = None
 		avatar_sha = None
-		# JEP-0172 User Nickname
+		# XEP-0172 User Nickname
 		user_nick = prs.getTagData('nick')
 		if not user_nick:
 			user_nick = ''
+		contact_nickname = None
 		transport_auto_auth = False
 		xtags = prs.getTags('x')
 		for x in xtags:
@@ -1566,11 +1607,12 @@ class ConnectionHandlers(ConnectionVcard, ConnectionBytestream, ConnectionDisco,
 				sigTag = x
 			if namespace == common.xmpp.NS_VCARD_UPDATE:
 				avatar_sha = x.getTagData('photo')
-			if namespace == common.xmpp.NS_DELAY:
-				# JEP-0091
+				contact_nickname = x.getTagData('nickname')
+			elif namespace == common.xmpp.NS_DELAY:
+				# XEP-0091
 				tim = prs.getTimestamp()
-				tim = time.strptime(tim, '%Y%m%dT%H:%M:%S')
-				timestamp = time.localtime(timegm(tim))
+				tim = helpers.datetime_tuple(tim)
+				timestamp = localtime(timegm(tim))
 			if namespace == 'http://delx.cjb.net/protocol/roster-subsync':
 				# see http://trac.gajim.org/ticket/326
 				agent = gajim.get_server_from_jid(jid_stripped)
@@ -1605,7 +1647,7 @@ class ConnectionHandlers(ConnectionVcard, ConnectionBytestream, ConnectionDisco,
 				errcode = prs.getErrorCode()
 				if errcode == '502': # Internal Timeout:
 					self.dispatch('NOTIFY', (jid_stripped, 'error', errmsg, resource,
-						prio, keyID, timestamp))
+						prio, keyID, timestamp, None))
 				elif errcode == '401': # password required to join
 					self.dispatch('ERROR', (_('Unable to join group chat'),
 						_('A password is required to join this group chat.')))
@@ -1638,7 +1680,8 @@ class ConnectionHandlers(ConnectionVcard, ConnectionBytestream, ConnectionDisco,
 			if not ptype or ptype == 'unavailable':
 				if gajim.config.get('log_contact_status_changes') and self.name\
 				not in no_log_for and jid_stripped not in no_log_for:
-					gc_c = gajim.contacts.get_gc_contact(self.name, jid_stripped, resource)
+					gc_c = gajim.contacts.get_gc_contact(self.name, jid_stripped,
+						resource)
 					st = status or ''
 					if gc_c:
 						jid = gc_c.jid
@@ -1647,34 +1690,21 @@ class ConnectionHandlers(ConnectionVcard, ConnectionBytestream, ConnectionDisco,
 					if jid:
 						# we know real jid, save it in db
 						st += ' (%s)' % jid
-					gajim.logger.write('gcstatus', who, st, show)
+					try:
+						gajim.logger.write('gcstatus', who, st, show)
+					except exceptions.PysqliteOperationalError, e:
+						self.dispatch('ERROR', (_('Disk Write Error'), str(e)))
 				if avatar_sha or avatar_sha == '':
 					if avatar_sha == '':
 						# contact has no avatar
 						puny_nick = helpers.sanitize_filename(resource)
 						gajim.interface.remove_avatar_files(jid_stripped, puny_nick)
-					if self.vcard_shas.has_key(who): # Verify sha cached in mem
-						if avatar_sha != self.vcard_shas[who]:
-							# avatar has been updated
-							self.request_vcard(who, True)
-					else: # Verify sha cached in hdd
-						cached_vcard = self.get_cached_vcard(who, True)
-						if cached_vcard and cached_vcard.has_key('PHOTO') and \
-						cached_vcard['PHOTO'].has_key('SHA'):
-							cached_sha = cached_vcard['PHOTO']['SHA']
-						else:
-							cached_sha = ''
-						if cached_sha != avatar_sha:
-							# avatar has been updated
-							# sha in mem will be updated later
-							self.request_vcard(who, True)
-						else:
-							# save sha in mem NOW
-							self.vcard_shas[who] = avatar_sha
+					# if it's a gc presence, don't ask vcard here. We may ask it to
+					# real jid in gui part.
 				self.dispatch('GC_NOTIFY', (jid_stripped, show, status, resource,
 					prs.getRole(), prs.getAffiliation(), prs.getJid(),
 					prs.getReason(), prs.getActor(), prs.getStatusCode(),
-					prs.getNewNick()))
+					prs.getNewNick(), avatar_sha))
 			return
 
 		if ptype == 'subscribe':
@@ -1687,7 +1717,7 @@ class ConnectionHandlers(ConnectionVcard, ConnectionBytestream, ConnectionDisco,
 					self.connection.send(p)
 				if who.find("@") <= 0 or transport_auto_auth:
 					self.dispatch('NOTIFY', (jid_stripped, 'offline', 'offline',
-						resource, prio, keyID, timestamp))
+						resource, prio, keyID, timestamp, None))
 				if transport_auto_auth:
 					self.automatically_added.append(jid_stripped)
 					self.request_subscription(jid_stripped, name = user_nick)
@@ -1702,10 +1732,10 @@ class ConnectionHandlers(ConnectionVcard, ConnectionBytestream, ConnectionDisco,
 				# detect a subscription loop
 				if not self.subscribed_events.has_key(jid_stripped):
 					self.subscribed_events[jid_stripped] = []
-				self.subscribed_events[jid_stripped].append(time.time())
+				self.subscribed_events[jid_stripped].append(time_time())
 				block = False
 				if len(self.subscribed_events[jid_stripped]) > 5:
-					if time.time() - self.subscribed_events[jid_stripped][0] < 5:
+					if time_time() - self.subscribed_events[jid_stripped][0] < 5:
 						block = True
 					self.subscribed_events[jid_stripped] = self.subscribed_events[jid_stripped][1:]
 				if block:
@@ -1722,10 +1752,10 @@ class ConnectionHandlers(ConnectionVcard, ConnectionBytestream, ConnectionDisco,
 			# detect a unsubscription loop
 			if not self.subscribed_events.has_key(jid_stripped):
 				self.subscribed_events[jid_stripped] = []
-			self.subscribed_events[jid_stripped].append(time.time())
+			self.subscribed_events[jid_stripped].append(time_time())
 			block = False
 			if len(self.subscribed_events[jid_stripped]) > 5:
-				if time.time() - self.subscribed_events[jid_stripped][0] < 5:
+				if time_time() - self.subscribed_events[jid_stripped][0] < 5:
 					block = True
 				self.subscribed_events[jid_stripped] = self.subscribed_events[jid_stripped][1:]
 			if block:
@@ -1738,7 +1768,7 @@ class ConnectionHandlers(ConnectionVcard, ConnectionBytestream, ConnectionDisco,
 			errcode = prs.getErrorCode()
 			if errcode == '502': # Internal Timeout:
 				self.dispatch('NOTIFY', (jid_stripped, 'error', errmsg, resource,
-					prio, keyID, timestamp))
+					prio, keyID, timestamp, None))
 			else:	# print in the window the error
 				self.dispatch('ERROR_ANSWER', ('', jid_stripped,
 					errmsg, errcode))
@@ -1755,73 +1785,18 @@ class ConnectionHandlers(ConnectionVcard, ConnectionBytestream, ConnectionDisco,
 				# avatar has been updated
 				self.request_vcard(jid_stripped)
 		if not ptype or ptype == 'unavailable':
-			if gajim.config.get('log_contact_status_changes') and self.name\
-				not in no_log_for and jid_stripped not in no_log_for:
-				gajim.logger.write('status', jid_stripped, status, show)
+			if gajim.config.get('log_contact_status_changes') and self.name \
+			not in no_log_for and jid_stripped not in no_log_for:
+				try:
+					gajim.logger.write('status', jid_stripped, status, show)
+				except exceptions.PysqliteOperationalError, e:
+					self.dispatch('ERROR', (_('Disk Write Error'), str(e)))
 			self.dispatch('NOTIFY', (jid_stripped, show, status, resource, prio,
-				keyID, timestamp))
+				keyID, timestamp, contact_nickname))
 	# END presenceCB
+
 	def _StanzaArrivedCB(self, con, obj):
 		self.last_io = gajim.idlequeue.current_time()
-	
-	
-	def parse_data_form(self, node):
-		dic = {}
-		tag = node.getTag('title')
-		if tag:
-			dic['title'] = tag.getData()
-		tag = node.getTag('instructions')
-		if tag:
-			dic['instructions'] = tag.getData()
-		i = 0
-		for child in node.getChildren():
-			if child.getName() != 'field':
-				continue
-			var = child.getAttr('var')
-			ctype = child.getAttr('type')
-			label = child.getAttr('label')
-			if not var and ctype != 'fixed': # We must have var if type != fixed
-				continue
-			dic[i] = {}
-			if var:
-				dic[i]['var'] = var
-			if ctype:
-				dic[i]['type'] = ctype
-			if label:
-				dic[i]['label'] = label
-			tags = child.getTags('value')
-			if len(tags):
-				dic[i]['values'] = []
-				for tag in tags:
-					data = tag.getData()
-					if ctype == 'boolean':
-						if data in ('yes', 'true', 'assent', '1'):
-							data = True
-						else:
-							data = False
-					dic[i]['values'].append(data)
-			tag = child.getTag('desc')
-			if tag:
-				dic[i]['desc'] = tag.getData()
-			option_tags = child.getTags('option')
-			if len(option_tags):
-				dic[i]['options'] = {}
-				j = 0
-				for option_tag in option_tags:
-					dic[i]['options'][j] = {}
-					label = option_tag.getAttr('label')
-					tags = option_tag.getTags('value')
-					dic[i]['options'][j]['values'] = []
-					for tag in tags:
-						dic[i]['options'][j]['values'].append(tag.getData())
-					if not label:
-						label = dic[i]['options'][j]['values'][0]
-					dic[i]['options'][j]['label'] = label
-					j += 1
-				if not dic[i].has_key('values'):
-					dic[i]['values'] = [dic[i]['options'][0]['values'][0]]
-			i += 1
-		return dic
 
 	def _MucOwnerCB(self, con, iq_obj):
 		gajim.log.debug('MucOwnerCB')
@@ -1832,8 +1807,7 @@ class ConnectionHandlers(ConnectionVcard, ConnectionBytestream, ConnectionDisco,
 				node = q
 		if not node:
 			return
-		dic = self.parse_data_form(node)
-		self.dispatch('GC_CONFIG', (helpers.get_full_jid_from_iq(iq_obj), dic))
+		self.dispatch('GC_CONFIG', (helpers.get_full_jid_from_iq(iq_obj), node))
 
 	def _MucAdminCB(self, con, iq_obj):
 		gajim.log.debug('MucAdminCB')
@@ -1873,6 +1847,8 @@ class ConnectionHandlers(ConnectionVcard, ConnectionBytestream, ConnectionDisco,
 		if not self.connection:
 			return
 		self.connection.getRoster(self._on_roster_set)
+		self.discoverItems(gajim.config.get_per('accounts', self.name, 
+			'hostname'), id_prefix='p')
 		if gajim.config.get_per('accounts', self.name, 'use_ft_proxies'):
 			self.discover_ft_proxies()
 	
@@ -1883,9 +1859,7 @@ class ConnectionHandlers(ConnectionVcard, ConnectionBytestream, ConnectionDisco,
 			proxies = map(lambda e:e.strip(), cfg_proxies.split(','))
 			for proxy in proxies:
 				gajim.proxy65_manager.resolve(proxy, self.connection)
-			self.discoverItems(gajim.config.get_per('accounts', self.name, 
-				'hostname'), id_prefix='p')
-	
+
 	def _on_roster_set(self, roster):
 		raw_roster = roster.getRaw()
 		roster = {}
