@@ -121,6 +121,7 @@ class P2PClient(IdleObject):
 			self.sock_type = TYPE_SERVER
 		else:
 			self.sock_type = TYPE_CLIENT
+		self.fd = -1
 		conn = P2PConnection('', _sock, host, port, self._caller, self.on_connect, self)
 		self.sock_hash = conn._sock.__hash__
 		self.fd = conn.fd
@@ -129,10 +130,14 @@ class P2PClient(IdleObject):
 		for val in self.stanzaqueue:
 			stanza, is_message = val
 			if is_message:
-				if self.conn_holder.number_of_awaiting_messages.has_key(self.fd):
-					self.conn_holder.number_of_awaiting_messages[self.fd]+=1
+				if self.fd == -1:
+					self._caller.dispatch('MSGERROR',[unicode(self.to), -1, \
+                                        _('Connection to host could not be established'), None, None])
 				else:
-					self.conn_holder.number_of_awaiting_messages[self.fd]=1
+					if self.conn_holder.number_of_awaiting_messages.has_key(self.fd):
+						self.conn_holder.number_of_awaiting_messages[self.fd]+=1
+					else:
+						self.conn_holder.number_of_awaiting_messages[self.fd]=1
 	
 	def add_stanza(self, stanza, is_message = False):
 		if self.Connection:
@@ -158,10 +163,6 @@ class P2PClient(IdleObject):
 		self.Connection.PlugIn(self)
 		dispatcher_nb.Dispatcher().PlugIn(self)
 		self._register_handlers()
-		if self.sock_type == TYPE_CLIENT:
-			while self.stanzaqueue:
-				stanza, is_message = self.stanzaqueue.pop(0)
-				self.send(stanza, is_message)
 
 	def StreamInit(self):
 		''' Send an initial stream header. '''
@@ -178,9 +179,11 @@ class P2PClient(IdleObject):
 	def send_stream_header(self):
 		self.Dispatcher._metastream = Node('stream:stream')
 		self.Dispatcher._metastream.setNamespace(self.Namespace)
-		# XXX TLS support
-		#~ self._metastream.setAttr('version', '1.0')
+		self.Dispatcher._metastream.setAttr('version', '1.0')
 		self.Dispatcher._metastream.setAttr('xmlns:stream', NS_STREAMS)
+		self.Dispatcher._metastream.setAttr('from', self.conn_holder.zeroconf.name)
+		if self.to:
+			self.Dispatcher._metastream.setAttr('to', self.to)
 		self.Dispatcher.send("<?xml version='1.0'?>%s>" % str(self.Dispatcher._metastream)[:-2])
 	
 	def _check_stream_start(self, ns, tag, attrs):
@@ -191,7 +194,17 @@ class P2PClient(IdleObject):
 			self.Connection.disconnect()
 			return
 		if self.sock_type == TYPE_SERVER:
+			if attrs.has_key('from'):
+				self.to = attrs['from']
 			self.send_stream_header()
+			if attrs.has_key('version') and attrs['version'] == '1.0':
+				# other part supports stream features
+				features = Node('stream:features')
+				self.Dispatcher.send(features)
+			while self.stanzaqueue:
+				stanza, is_message = self.stanzaqueue.pop(0)
+				self.send(stanza, is_message)
+		elif self.sock_type == TYPE_CLIENT:
 			while self.stanzaqueue:
 				stanza, is_message = self.stanzaqueue.pop(0)
 				self.send(stanza, is_message)
@@ -199,11 +212,11 @@ class P2PClient(IdleObject):
 	
 	def on_disconnect(self):
 		if self.conn_holder:
-			if self.conn_holder.number_of_awaiting_messages.has_key(self.fd):
-				if self.conn_holder.number_of_awaiting_messages[self.fd] > 0:
+			if self.conn_holder.number_of_awaiting_messages.has_key(self.conn_holder.fd):
+				if self.conn_holder.number_of_awaiting_messages[self.conn_holder.fd] > 0:
 					self._caller.dispatch('MSGERROR',[unicode(self.to), -1, \
 					_('Connection to host could not be established'), None, None])
-				del self.conn_holder.number_of_awaiting_messages[self.fd]
+				del self.conn_holder.number_of_awaiting_messages[self.conn_holder.fd]
 			self.conn_holder.remove_connection(self.sock_hash) 
 		if self.__dict__.has_key('Dispatcher'):
 			self.Dispatcher.PlugOut()
@@ -310,8 +323,9 @@ class P2PConnection(IdleObject, PlugIn):
 		if not recv_handler(None) and _tmp == self.on_receive:
 			self.on_receive = recv_handler
 		
-	def send(self, packet, is_message = False):
-		'''Append stanza to the queue of messages to be send. 
+	def send(self, packet, is_message = False, now = False):
+		'''Append stanza to the queue of messages to be send if now is
+		False, else send it instantly. 
 		If supplied data is unicode string, encode it to utf-8.
 		'''
 		if self.state <= 0:
@@ -324,7 +338,11 @@ class P2PConnection(IdleObject, PlugIn):
 		elif not isinstance(r, str): 
 			r = ustr(r).encode('utf-8')
 
-		self.sendqueue.append((r, is_message))
+		if now:
+			self.sendqueue.insert(0, (r, is_message))
+			self._do_send()
+		else:
+			self.sendqueue.append((r, is_message))
 		self._plug_idle()
 		
 	def read_timeout(self):
@@ -619,7 +637,7 @@ class ClientZeroconf:
 			return self.roster.getRoster()
 		return {}
 
-	def send(self, stanza, is_message = False):
+	def send(self, stanza, is_message = False, now = False):
 		stanza.setFrom(self.roster.zeroconf.name)
 		to = stanza.getTo()
 		
