@@ -52,6 +52,7 @@ class ConnectionZeroconf(ConnectionHandlersZeroconf):
 		# system username
 		self.username = None
 		self.name = name
+		self.server_resource = '' # zeroconf has no resource, fake an empty one
 		self.connected = 0 # offline
 		self.connection = None
 		self.gpg = None
@@ -63,7 +64,7 @@ class ConnectionZeroconf(ConnectionHandlersZeroconf):
 	
 		self.call_resolve_timeout = False
 		
-		#self.time_to_reconnect = None
+		self.time_to_reconnect = None
 		#self.new_account_info = None
 		self.bookmarks = []
 
@@ -86,6 +87,7 @@ class ConnectionZeroconf(ConnectionHandlersZeroconf):
 		
 		self.muc_jid = {} # jid of muc server for each transport type
 		self.vcard_supported = False
+		self.private_storage_supported = False
 
 	def get_config_values_or_default(self):
 		''' get name, host, port from config, or 
@@ -99,12 +101,11 @@ class ConnectionZeroconf(ConnectionHandlersZeroconf):
 			gajim.config.set_per('accounts', gajim.ZEROCONF_ACC_NAME, 'password', 'zeroconf')
 			gajim.config.set_per('accounts', gajim.ZEROCONF_ACC_NAME, 'sync_with_global_status', True)
 
-			#XXX make sure host is US-ASCII
-			self.host = unicode(socket.gethostname())
-			gajim.config.set_per('accounts', gajim.ZEROCONF_ACC_NAME, 'hostname', self.host)
 			gajim.config.set_per('accounts', gajim.ZEROCONF_ACC_NAME, 'custom_port', 5298)
 			gajim.config.set_per('accounts', gajim.ZEROCONF_ACC_NAME, 'is_zeroconf', True)
-		self.host = gajim.config.get_per('accounts', gajim.ZEROCONF_ACC_NAME, 'hostname')
+		#XXX make sure host is US-ASCII
+		self.host = unicode(socket.gethostname())
+		gajim.config.set_per('accounts', gajim.ZEROCONF_ACC_NAME, 'hostname', self.host)
 		self.port = gajim.config.get_per('accounts', gajim.ZEROCONF_ACC_NAME, 'custom_port')
 		self.autoconnect = gajim.config.get_per('accounts', gajim.ZEROCONF_ACC_NAME, 'autoconnect')
 		self.sync_with_global_status = gajim.config.get_per('accounts', gajim.ZEROCONF_ACC_NAME, 'sync_with_global_status')
@@ -126,10 +127,12 @@ class ConnectionZeroconf(ConnectionHandlersZeroconf):
 			gajim.handlers[event](self.name, data)
 
 	def _reconnect(self):
+		# Do not try to reco while we are already trying
+		self.time_to_reconnect = None
 		gajim.log.debug('reconnect')
 
-		signed = self.get_signed_msg(self.status)
-		self.reconnect()
+#		signed = self.get_signed_msg(self.status)
+		self.connect(self.old_show, self.status)
 
 	def quit(self, kill_core):
 		if kill_core and self.connected > 1:
@@ -198,6 +201,19 @@ class ConnectionZeroconf(ConnectionHandlersZeroconf):
 		self.status = 'offline'
 		self.disconnect()
 
+	def _disconnectedReconnCB(self):
+		'''Called when we are disconnected. Comes from network manager for example
+		we don't try to reconnect, network manager will tell us when we can'''
+		if gajim.account_is_connected(self.name):
+			# we cannot change our status to offline or connecting
+			# after we auth to server
+			self.old_show = STATUS_LIST[self.connected]
+		self.connected = 0
+		self.dispatch('STATUS', 'offline')
+		# random number to show we wait network manager to send us a reconenct
+		self.time_to_reconnect = 5
+		self.on_purpose = False
+
 	def _on_name_conflictCB(self, alt_name):
 		self.disconnect()
 		self.dispatch('STATUS', 'offline')
@@ -299,6 +315,7 @@ class ConnectionZeroconf(ConnectionHandlersZeroconf):
 		# 'disconnect'
 		elif show == 'offline' and self.connected:
 			self.disconnect()
+			self.time_to_reconnect = None
 			
 		# update status
 		elif show != 'offline' and self.connected:
@@ -328,15 +345,15 @@ class ConnectionZeroconf(ConnectionHandlersZeroconf):
 		return STATUS_LIST[self.connected]
 
 	def send_message(self, jid, msg, keyID, type = 'chat', subject='',
-	chatstate = None, msg_id = None, composing_jep = None, resource = None, 
+	chatstate = None, msg_id = None, composing_xep = None, resource = None, 
 	user_nick = None):
 		fjid = jid
-		
+
 		if not self.connection:
 			return
 		if not msg and chatstate is None:
 			return
-		
+
 		if self.status in ('invisible', 'offline'):
 			self.dispatch('MSGERROR', [unicode(jid), '-1', _('You are not connected or not visible to others. Your message could not be sent.'), None, None])
 			return
@@ -344,19 +361,23 @@ class ConnectionZeroconf(ConnectionHandlersZeroconf):
 		msgtxt = msg
 		msgenc = ''
 		if keyID and USE_GPG:
-			#encrypt
-			msgenc = self.gpg.encrypt(msg, [keyID])
-			if msgenc:
+			# encrypt
+			msgenc, error = self.gpg.encrypt(msg, [keyID])
+			if msgenc and not error:
 				msgtxt = '[This message is encrypted]'
 				lang = os.getenv('LANG')
 				if lang is not None or lang != 'en': # we're not english
 					msgtxt = _('[This message is encrypted]') +\
 						' ([This message is encrypted])' # one  in locale and one en
+			else:
+				# Encryption failed, do not send message
+				tim = time.localtime()
+				self.dispatch('MSGNOTSENT', (jid, error, msgtxt, tim))
+				return 3
 
-		
 		if type == 'chat':
 			msg_iq = common.xmpp.Message(to = fjid, body = msgtxt, typ = type)
-			
+
 		else:
 			if subject:
 				msg_iq = common.xmpp.Message(to = fjid, body = msgtxt,
@@ -367,15 +388,15 @@ class ConnectionZeroconf(ConnectionHandlersZeroconf):
 
 		if msgenc:
 			msg_iq.setTag(common.xmpp.NS_ENCRYPTED + ' x').setData(msgenc)
-		
+
 		# chatstates - if peer supports jep85 or jep22, send chatstates
 		# please note that the only valid tag inside a message containing a <body>
 		# tag is the active event
 		if chatstate is not None:
-			if composing_jep == 'JEP-0085' or not composing_jep:
+			if composing_xep == 'XEP-0085' or not composing_xep:
 				# JEP-0085
 				msg_iq.setTag(chatstate, namespace = common.xmpp.NS_CHATSTATES)
-			if composing_jep == 'JEP-0022' or not composing_jep:
+			if composing_xep == 'XEP-0022' or not composing_xep:
 				# JEP-0022
 				chatstate_node = msg_iq.setTag('x', namespace = common.xmpp.NS_EVENT)
 				if not msgtxt: # when no <body>, add <id>
@@ -385,7 +406,7 @@ class ConnectionZeroconf(ConnectionHandlersZeroconf):
 				# when msgtxt, requests JEP-0022 composing notification
 				if chatstate is 'composing' or msgtxt: 
 					chatstate_node.addChild(name = 'composing') 
-		
+
 		if not self.connection.send(msg_iq, msg != None):
 			return
 
