@@ -1047,7 +1047,7 @@ class ToplevelAgentBrowser(AgentBrowser):
 		# as it was before setting the timeout
 		if props and self.tooltip.id == props[0]:
 			# bounding rectangle of coordinates for the cell within the treeview
-			rect =  view.get_cell_area(props[0], props[1])
+			rect = view.get_cell_area(props[0], props[1])
 			# position of the treeview on the screen
 			position = view.window.get_origin()
 			self.tooltip.show_tooltip(state, rect.height, position[1] + rect.y)
@@ -1487,10 +1487,10 @@ class MucBrowser(AgentBrowser):
 		self.join_button = None
 
 	def _create_treemodel(self):
-		# JID, node, name, users, description, fetched
+		# JID, node, name, users_int, users_str, description, fetched
 		# This is rather long, I'd rather not use a data_func here though.
 		# Users is a string, because want to be able to leave it empty.
-		self.model = gtk.ListStore(str, str, str, str, str, bool)
+		self.model = gtk.ListStore(str, str, str, int, str, str, bool)
 		self.model.set_sort_column_id(2, gtk.SORT_ASCENDING)
 		self.window.services_treeview.set_model(self.model)
 		# Name column
@@ -1500,20 +1500,23 @@ class MucBrowser(AgentBrowser):
 		renderer = gtk.CellRendererText()
 		col.pack_start(renderer)
 		col.set_attributes(renderer, text = 2)
+		col.set_sort_column_id(2)
 		self.window.services_treeview.insert_column(col, -1)
 		col.set_resizable(True)
 		# Users column
 		col = gtk.TreeViewColumn(_('Users'))
 		renderer = gtk.CellRendererText()
 		col.pack_start(renderer)
-		col.set_attributes(renderer, text = 3)
+		col.set_attributes(renderer, text = 4)
+		col.set_sort_column_id(3)
 		self.window.services_treeview.insert_column(col, -1)
 		col.set_resizable(True)
 		# Description column
 		col = gtk.TreeViewColumn(_('Description'))
 		renderer = gtk.CellRendererText()
 		col.pack_start(renderer)
-		col.set_attributes(renderer, text = 4)
+		col.set_attributes(renderer, text = 5)
+		col.set_sort_column_id(4)
 		self.window.services_treeview.insert_column(col, -1)
 		col.set_resizable(True)
 		# Id column
@@ -1521,9 +1524,11 @@ class MucBrowser(AgentBrowser):
 		renderer = gtk.CellRendererText()
 		col.pack_start(renderer)
 		col.set_attributes(renderer, text = 0)
+		col.set_sort_column_id(0)
 		self.window.services_treeview.insert_column(col, -1)
 		col.set_resizable(True)
 		self.window.services_treeview.set_headers_visible(True)
+		self.window.services_treeview.set_headers_clickable(True)
 		# Source id for idle callback used to start disco#info queries.
 		self._fetch_source = None
 		# Query failure counter
@@ -1625,7 +1630,7 @@ class MucBrowser(AgentBrowser):
 			# We're at the end of the model, we can leave end=None though.
 			pass
 		while iter and self.model.get_path(iter) != end:
-			if not self.model.get_value(iter, 5):
+			if not self.model.get_value(iter, 6):
 				jid = self.model.get_value(iter, 0).decode('utf-8')
 				node = self.model.get_value(iter, 1).decode('utf-8')
 				self.cache.get_info(jid, node, self._agent_info)
@@ -1656,13 +1661,14 @@ class MucBrowser(AgentBrowser):
 			if iter:
 				if name:
 					self.model[iter][2] = name
-				self.model[iter][3] = len(items)		# The number of users
-				self.model[iter][5] = True
+				self.model[iter][3] = len(items) # The number of users
+				self.model[iter][4] = str(len(items)) # The number of users
+				self.model[iter][6] = True
 		self._fetch_source = None
 		self._query_visible()
 
 	def _add_item(self, jid, node, item, force):
-		self.model.append((jid, node, item.get('name', ''), '', '', False))
+		self.model.append((jid, node, item.get('name', ''), -1, '', '', False))
 		if not self._fetch_source:
 			self._fetch_source = gobject.idle_add(self._start_info_query)
 
@@ -1676,14 +1682,15 @@ class MucBrowser(AgentBrowser):
 				users = form.getField('muc#roominfo_occupants')
 				descr = form.getField('muc#roominfo_description')
 				if users:
-					self.model[iter][3] = users.getValue()
+					self.model[iter][3] = int(users.getValue())
+					self.model[iter][4] = users.getValue()
 				if descr:
-					self.model[iter][4] = descr.getValue()
+					self.model[iter][5] = descr.getValue()
 				# Only set these when we find a form with additional info
 				# Some servers don't support forms and put extra info in
 				# the name attribute, so we preserve it in that case.
 				self.model[iter][2] = name
-				self.model[iter][5] = True
+				self.model[iter][6] = True
 				break
 		else:
 			# We didn't find a form, switch to alternate query mode
@@ -1737,6 +1744,7 @@ class DiscussionGroupsBrowser(AgentBrowser):
 		col.set_attributes(renderer, markup=2)
 		col.set_resizable(True)
 		self.window.services_treeview.insert_column(col, -1)
+		self.window.services_treeview.set_headers_visible(True)
 
 		# Subscription state
 		renderer = gtk.CellRendererToggle()
@@ -1838,7 +1846,7 @@ class DiscussionGroupsBrowser(AgentBrowser):
 		model, iter = self.window.services_treeview.get_selection().get_selected()
 		if iter is None: return
 
-		groupnode = model.get_value(iter, 1)    # 1 = groupnode
+		groupnode = model.get_value(iter, 1) # 1 = groupnode
 		
 		gajim.connections[self.account].send_pb_unsubscribe(self.jid, groupnode, self._unsubscribeCB, groupnode)
 
