@@ -208,7 +208,7 @@ def pid_alive():
 					return b.contents.szExeFile
 				f = k.Process32Next(h, b)
 
-		if get_p(pid) == 'python.exe':
+		if get_p(pid) in ('python.exe', 'gajim.exe'):
 			return True
 		return False
 	try:
@@ -257,9 +257,15 @@ pid_dir =  os.path.dirname(pid_filename)
 if not os.path.exists(pid_dir):
 	check_paths.create_path(pid_dir)
 # Create pid file
-f = open(pid_filename, 'w')
-f.write(str(os.getpid()))
-f.close()
+try:
+	f = open(pid_filename, 'w')
+	f.write(str(os.getpid()))
+	f.close()
+except IOError, e:
+	dlg = dialogs.ErrorDialog(_('Disk Write Error'), str(e))
+	dlg.run()
+	dlg.destroy()
+	sys.exit()
 del pid_dir
 del f
 
@@ -342,13 +348,16 @@ class Interface:
 			gc_control.show_change_nick_input_dialog(title, prompt, proposed_nick)
 
 	def handle_event_http_auth(self, account, data):
-		#('HTTP_AUTH', account, (method, url, transaction_id, iq_obj))
+		#('HTTP_AUTH', account, (method, url, transaction_id, iq_obj, msg))
 		def response(widget, account, iq_obj, answer):
 			self.dialog.destroy()
 			gajim.connections[account].build_http_auth_answer(iq_obj, answer)
 
+		sec_msg = _('Do you accept this request?')
+		if data[4]:
+			sec_msg = data[4] + '\n' + sec_msg
 		self.dialog = dialogs.YesNoDialog(_('HTTP (%s) Authorization for %s (id: %s)') \
-			% (data[0], data[1], data[2]), _('Do you accept this request?'),
+			% (data[0], data[1], data[2]), sec_msg,
 			on_response_yes = (response, account, data[3], 'yes'),
 			on_response_no = (response, account, data[3], 'no'))
 
@@ -444,7 +453,7 @@ class Interface:
 
 	def handle_event_notify(self, account, array):
 		# 'NOTIFY' (account, (jid, status, status message, resource, priority,
-		# keyID, timestamp))
+		# keyID, timestamp, contact_nickname))
 		# if we're here it means contact changed show
 		statuss = ['offline', 'error', 'online', 'chat', 'away', 'xa', 'dnd',
 			'invisible']
@@ -456,6 +465,7 @@ class Interface:
 		status_message = array[2]
 		jid = array[0].split('/')[0]
 		keyID = array[5]
+		contact_nickname = array[7]
 		attached_keys = gajim.config.get_per('accounts', account,
 			'attached_gpg_keys').split()
 		if jid in attached_keys:
@@ -484,6 +494,10 @@ class Interface:
 			if contact1:
 				if contact1.show in statuss:
 					old_show = statuss.index(contact1.show)
+				if contact_nickname is not None and \
+				contact1.contact_name != contact_nickname:
+					contact1.contact_name = contact_nickname
+					self.roster.draw_contact(jid, account)
 				if old_show == new_show and contact1.status == status_message and \
 					contact1.priority == priority: # no change
 					return
@@ -538,6 +552,7 @@ class Interface:
 			elif not gajim.block_signed_in_notifications[account]:
 				# We're connected since more that 30 seconds
 				contact1.last_status_time = time.localtime()
+			contact1.contact_nickname = contact_nickname
 		if gajim.jid_is_transport(jid):
 			# It must be an agent
 			if ji in jid_list:
@@ -569,7 +584,7 @@ class Interface:
 			# (when contact signs out or has errors)
 			if array[1] in ('offline', 'error'):
 				contact1.our_chatstate = contact1.chatstate = \
-					contact1.composing_jep = None
+					contact1.composing_xep = None
 				gajim.connections[account].remove_transfers_for_contact(contact1)
 			self.roster.chg_contact_status(contact1, array[1], status_message,
 				account)
@@ -598,7 +613,7 @@ class Interface:
 
 	def handle_event_msg(self, account, array):
 		# 'MSG' (account, (jid, msg, time, encrypted, msg_type, subject,
-		# chatstate, msg_id, composing_jep, user_nick, xhtml))
+		# chatstate, msg_id, composing_xep, user_nick, xhtml))
 		# user_nick is JEP-0172
 
 		full_jid_with_resource = array[0]
@@ -611,7 +626,7 @@ class Interface:
 		subject = array[5]
 		chatstate = array[6]
 		msg_id = array[7]
-		composing_jep = array[8]
+		composing_xep = array[8]
 		xhtml = array[10]
 		if gajim.config.get('ignore_incoming_xhtml'):
 			xhtml = None
@@ -652,8 +667,8 @@ class Interface:
 		if contact and isinstance(contact, list):
 			contact = contact[0]
 		if contact:
-			if contact.composing_jep != 'JEP-0085': # We cache xep85 support
-				contact.composing_jep = composing_jep
+			if contact.composing_xep != 'XEP-0085': # We cache xep85 support
+				contact.composing_xep = composing_xep
 			if chat_control and chat_control.type_id == message_control.TYPE_CHAT:
 				if chatstate is not None:
 					# other peer sent us reply, so he supports jep85 or jep22
@@ -698,7 +713,7 @@ class Interface:
 		if pm:
 			nickname = resource
 			groupchat_control.on_private_message(nickname, message, array[2],
-				xhtml)
+				xhtml, msg_id)
 		else:
 			# array: (jid, msg, time, encrypted, msg_type, subject)
 			if encrypted:
@@ -743,8 +758,7 @@ class Interface:
 						show = 'offline'
 					gc_c = gajim.contacts.create_gc_contact(room_jid = jid,
 						name = nick, show = show)
-					c = gajim.contacts.contact_from_gc_contact(gc_c)
-					self.roster.new_chat(c, account, private_chat = True)
+					self.roster.new_private_chat(gc_c, account)
 				ctrl = self.msg_win_mgr.get_control(full_jid_with_resource, account)
 				ctrl.print_conversation('Error %s: %s' % (array[1], array[2]),
 							'status')
@@ -853,8 +867,9 @@ class Interface:
 				self.roster.remove_contact(c, account)
 
 	def handle_event_register_agent_info(self, account, array):
-		#('REGISTER_AGENT_INFO', account, (agent, infos, is_form))
-		if array[1].has_key('instructions'):
+		# ('REGISTER_AGENT_INFO', account, (agent, infos, is_form))
+		# info in a dataform if is_form is True
+		if array[2] or array[1].has_key('instructions'):
 			config.ServiceRegistrationWindow(array[0], array[1], account,
 				array[2])
 		else:
@@ -895,15 +910,18 @@ class Interface:
 
 	def handle_event_myvcard(self, account, array):
 		nick = ''
-		if array.has_key('NICKNAME'):
-			nick = array['NICKNAME']
-			if nick:
-				gajim.nicks[account] = nick
+		if array.has_key('NICKNAME') and array['NICKNAME']:
+			gajim.nicks[account] = array['NICKNAME']
+		elif array.has_key('FN') and array['FN']:
+			gajim.nicks[account] = array['FN']
 		if self.instances[account].has_key('profile'):
 			win = self.instances[account]['profile']
 			win.set_values(array)
 			if account in self.show_vcard_when_connect:
 				self.show_vcard_when_connect.remove(account)
+		jid = array['jid']
+		if self.instances[account]['infos'].has_key(jid):
+			self.instances[account]['infos'][jid].set_values(array)
 
 	def handle_event_vcard(self, account, vcard):
 		# ('VCARD', account, data)
@@ -981,7 +999,7 @@ class Interface:
 
 	def handle_event_gc_notify(self, account, array):
 		#'GC_NOTIFY' (account, (room_jid, show, status, nick,
-		# role, affiliation, jid, reason, actor, statusCode, newNick))
+		# role, affiliation, jid, reason, actor, statusCode, newNick, avatar_sha))
 		nick = array[3]
 		if not nick:
 			return
@@ -997,7 +1015,7 @@ class Interface:
 			return
 		if control:
 			control.chg_contact_status(nick, show, status, array[4], array[5],
-				array[6], array[7], array[8], array[9], array[10])
+				array[6], array[7], array[8], array[9], array[10], array[11])
 
 		# print status in chat window and update status/GPG image
 		if self.msg_win_mgr.has_window(fjid, account):
@@ -1061,7 +1079,7 @@ class Interface:
 				gc_control.print_conversation(text)
 
 	def handle_event_gc_config(self, account, array):
-		#('GC_CONFIG', account, (jid, config))  config is a dict
+		#('GC_CONFIG', account, (jid, form))  config is a dict
 		room_jid = array[0].split('/')[0]
 		if room_jid in gajim.automatic_rooms[account]:
 			# use default configuration
@@ -1616,16 +1634,20 @@ class Interface:
 				gajim.status_before_autoaway[account] = \
 					gajim.connections[account].status
 				# we go away (no auto status) [we pass True to auto param]
-				self.roster.send_status(account, 'away',
-					gajim.config.get('autoaway_message'), auto=True)
+				auto_message = gajim.config.get('autoaway_message')
+				if not auto_message:
+					auto_message = gajim.connections[account].status
+				self.roster.send_status(account, 'away', auto_message, auto=True)
 				gajim.sleeper_state[account] = 'autoaway'
 			elif state == common.sleepy.STATE_XA and (\
 				gajim.sleeper_state[account] == 'autoaway' or \
 				gajim.sleeper_state[account] == 'online') and \
 				gajim.config.get('autoxa'):
 				# we go extended away [we pass True to auto param]
-				self.roster.send_status(account, 'xa',
-					gajim.config.get('autoxa_message'), auto=True)
+				auto_message = gajim.config.get('autoxa_message')
+				if not auto_message:
+					auto_message = gajim.connections[account].status
+				self.roster.send_status(account, 'xa', auto_message, auto=True)
 				gajim.sleeper_state[account] = 'autoxa'
 		return True # renew timeout (loop for ever)
 
@@ -1976,6 +1998,8 @@ class Interface:
 			if not event:
 				# default to jid without resource
 				event = gajim.events.get_first_event(account, jid, type_)
+				if not event:
+					return
 				# Open the window
 				self.roster.open_event(account, jid, event)
 			else:
@@ -2034,7 +2058,7 @@ class Interface:
 		#add default themes if there is not in the config file
 		theme = gajim.config.get('roster_theme')
 		if not theme in gajim.config.get_per('themes'):
-			gajim.config.set('roster_theme', 'gtk+')
+			gajim.config.set('roster_theme', _('default'))
 		if len(gajim.config.get_per('themes')) == 0:
 			d = ['accounttextcolor', 'accountbgcolor', 'accountfont',
 				'accountfontattrs', 'grouptextcolor', 'groupbgcolor', 'groupfont',
@@ -2050,7 +2074,8 @@ class Interface:
 					gajim.config.set_per('themes', theme_name, o,
 						theme[d.index(o)])
 			
-		if gajim.config.get('autodetect_browser_mailer'):
+		if gajim.config.get('autodetect_browser_mailer') or \
+		len(gajim.connections) == 0:
 			gtkgui_helpers.autodetect_browser_mailer()
 
 		if gajim.verbose:
@@ -2116,9 +2141,8 @@ class Interface:
 			self.remote_ctrl = None
 
 		if gajim.config.get('networkmanager_support') and dbus_support.supported:
-			try:
-				import network_manager_listener
-			except:
+			import network_manager_listener
+			if not network_manager_listener.supported:
 				print >> sys.stderr, _('Network Manager support not available')
 
 		self.show_vcard_when_connect = []
@@ -2180,8 +2204,11 @@ class Interface:
 		self.last_ftwindow_update = 0
 
 		gobject.timeout_add(100, self.autoconnect)
-		gobject.timeout_add(200, self.process_connections)
-		gobject.timeout_add(500, self.read_sleepy)
+		if os.name == 'nt':
+			gobject.timeout_add(200, self.process_connections)
+		else:
+			gobject.timeout_add(2000, self.process_connections)
+		gobject.timeout_add(10000, self.read_sleepy)
 
 if __name__ == '__main__':
 	def sigint_cb(num, stack):
